import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { Meta, Title } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { DateTime } from 'luxon';
import { lastValueFrom } from 'rxjs';
import { DataType } from '../models/DataType.model';
import { Structure } from '../models/structure.model';
import { Week, WeekDayEnum } from '../models/week.model';
import { AuthService } from '../services/auth.service';
import { ConfigService } from '../services/config.service';
import { DataService } from '../services/data.service';
import { NotificationService } from '../services/notification.service';
import { Category } from '../structure-list/models/category.model';
import { Module } from '../structure-list/models/module.model';
import { SearchService } from '../structure-list/services/search.service';
import { Utils } from '../utils/utils';
import { territoryOptions } from './enums/territoryOptions.enum';

@Component({
  selector: 'app-data',
  templateUrl: './data.component.html',
  styleUrls: ['./data.component.scss'],
})
export class DataComponent implements OnInit {
  public structures = this.dataService.structures;
  public users = this.dataService.users;
  public categoriesCTMs: Category[];
  public categoriesInseeCodes: Category[];
  public categoriesTerritory: Category[];
  public checkedModulesFilter: Module[] = [];
  public checkedModulesFilterLabel: string;
  public checkedModulesCTMs: Module[] = [];
  public checkedModulesInseeCodes: Module[] = [];
  public checkedModulesTerritory: Module[] = [];
  public territorySelectLabel = 'Sélectionnez un territoire';
  public hasResults = false;
  public selectedOption = '';
  public isDownloading = false;

  private utils = new Utils();
  private categories: Category[];

  constructor(
    private authService: AuthService,
    private dataService: DataService,
    public searchService: SearchService,
    private notificationService: NotificationService,
    private http: HttpClient,
    private router: Router,
    private title: Title,
    private meta: Meta,
    private configService: ConfigService,
  ) {}

  ngOnInit(): void {
    this.searchService.getCategories().subscribe((categories) => {
      this.categoriesTerritory = [
        {
          id: 'Territory',
          name: 'Territory',
          theme: 'Territory',
          modules: Object.keys(territoryOptions)
            .filter((key) => !isNaN(Number(key)))
            .map((key, index) => ({
              disabled: false,
              name: territoryOptions[key],
              id: 'choice' + (index + 1).toString(),
            })),
        },
      ];
      this.categories = categories;
      this.categoriesCTMs = categories.filter((c) => c.id === 'ctm');
    });

    // Get cities from data.grandlyon.com
    // To change in proxy.conf.json and default.conf to have new city Oullins-Pierre-Bénite :
    // /fr/datapusher/ws/grandlyon/adr_voie_lieu.adrcomgl_2024/all.json
    this.http.get('/data-grandlyon-cities', { headers: { skip: 'true' } }).subscribe((data: { values: any[] }) => {
      this.categoriesInseeCodes = [
        {
          id: 'inseeCode',
          name: 'Commune',
          theme: 'Commune',
          modules: data.values
            .flatMap((value) => {
              if (value.nom === 'Lyon') {
                // Replace city "Lyon" by the 9 districts of Lyon
                return Array.from({ length: 9 }, (_, index) => ({
                  disabled: false,
                  name: `Lyon ${index + 1}`,
                  id: `6938${index + 1}`, // Insee code for district of Lyon
                }));
              }
              return [{ disabled: false, name: value.nom, id: value.insee }];
            })
            .sort((a, b) => a.name.localeCompare(b.name)),
        },
      ];
    });

    this.resetFilters();
    this.configService.getConfig().then((config) => {
      this.setMetaTags(config.appUrl);
    });
  }

  public async downloadStructurePdf(dataType: DataType): Promise<void> {
    this.isDownloading = true;
    const fileName = this.getFileBasename(dataType) + '.pdf';

    let urlPath = 'donnees/structures?data=' + dataType.slug;
    if (this.selectedOption === 'choice2') {
      const CTMs = this.categoriesCTMs[0].modules.map((ctm) => ctm.id).join(',');
      urlPath += '&ctm=' + CTMs;
    } else if (this.checkedModulesCTMs.length) {
      const CTMs = this.checkedModulesCTMs.map((m) => m.id).join(',');
      urlPath += '&ctm=' + CTMs;
    }
    if (this.checkedModulesInseeCodes.length) {
      const inseeCodes = this.checkedModulesInseeCodes.map((m) => m.id).join(',');
      urlPath += '&inseeCode=' + inseeCodes;
    }

    const pdfBlob = await lastValueFrom(this.dataService.getPdf(urlPath, fileName)).catch(() => {
      this.notificationService.showErrorPleaseRetry('Échec de la création du fichier pdf');
      return;
    });
    if (!pdfBlob) {
      this.isDownloading = false;
      return;
    }

    // Create a temporary URL for downloading
    const blobUrl = window.URL.createObjectURL(pdfBlob);
    const link = document.createElement('a');
    link.href = blobUrl;
    link.download = fileName;

    // Start the download
    link.click();
    this.isDownloading = false;

    // Clean up the temporary URL after the download
    window.URL.revokeObjectURL(blobUrl);
  }

  public async downloadStructureCsv(dataType: DataType): Promise<void> {
    this.isDownloading = true;
    const CTMs = this.getCTMs();
    const inseeCodes = this.checkedModulesInseeCodes.map((m) => m.id);

    // Get structures using specified filter for this DataType
    const structures = await lastValueFrom(this.dataService.getStructures(dataType, CTMs, inseeCodes)).catch(() => {
      this.notificationService.showErrorPleaseRetry('Échec de la création du fichier csv');
      return;
    });
    if (!structures) {
      this.isDownloading = false;
      return;
    }

    if (structures.length === 0) {
      this.notificationService.showError(
        'Aucune structure ne correspond à vos paramètres',
        'Veuillez modifier votre recherche',
        10000,
      );
      this.isDownloading = false;
      return;
    }

    // Sort for csv file
    structures.sort(
      (a, b) => a.address.commune.localeCompare(b.address.commune) || a.structureName.localeCompare(b.structureName),
    );

    // Filter fields in structures
    const filteredStructures = structures.map((structure) => {
      // Set category names in this.structure.categoriesDisplay for structure and all its personal offers
      this.utils.setServiceCategoriesWithPersonalOffers(this.categories, structure);

      return {
        Nom: structure.structureName,
        Commune: structure.address.commune,
        Adresse: `${structure.address.numero ?? ''} ${structure.address.street}`.trim(),
        'Aides aux démarches en ligne (gratuits)': structure.categoriesDisplay?.onlineProcedures
          .map((acc) => acc.name)
          .join(', '),
        'Accompagnements aux usages numériques': structure.categoriesDisplay?.baseSkills
          .concat(structure.categoriesDisplay?.advancedSkills)
          .map((skill) => skill.name)
          .join(', '),
        'Gratuité des accompagnements':
          structure.categoriesDisplay?.freeWorkShop?.length > 0 ? structure.categoriesDisplay.freeWorkShop[0].name : '',
        Téléphone: structure.contactPhone || '',
        Email: structure.contactMail || '',
        Description: structure.description || '',
        Horaires: this.formatOpeningHoursForCSV(structure.hours),
        labels: structure.categoriesDisplay?.labelsQualifications.map((label) => label.name).join(', '),
        "Modalité d'accès": structure.categoriesDisplay?.accessModality.map((access) => access.name).join(', '),
        'Publics accueillis': structure.categoriesDisplay?.age
          .map((concernedPeople) => concernedPeople.name)
          .join(', '),
        'Date de mise à jour': DateTime.fromISO(structure.updatedAt).toFormat('yyyy-MM-dd HH:mm'),
      };
    });

    // Convert and download to csv
    this.utils.convertAndDownloadCsv(filteredStructures, this.getFileBasename(dataType) + '.csv');

    this.isDownloading = false;
  }

  public async downloadUserCsv(dataType: DataType): Promise<void> {
    // Check if the user is logged in
    if (!this.authService.isLoggedIn()) {
      this.notificationService.showError(
        'Utilisation interne uniquement',
        'Pour accéder aux membres, veuillez vous connecter',
        10000,
      );
      return;
    }

    this.isDownloading = true;
    const CTMs = this.getCTMs();
    const inseeCodes = this.checkedModulesInseeCodes.map((m) => m.id);

    // Get users using specified filter for this DataType
    const users = await this.dataService.getUsers(dataType, CTMs, inseeCodes).catch(() => {
      this.notificationService.showErrorPleaseRetry('Échec de la création du fichier csv');
      return;
    });
    if (!users) {
      this.isDownloading = false;
      return;
    }

    if (users.length === 0) {
      this.notificationService.showError(
        'Aucun membre ne correspond à vos paramètres',
        'Veuillez modifier votre recherche',
        10000,
      );
      this.isDownloading = false;
      return;
    }

    // Sort for csv file
    users.sort((a, b) => a.surname.localeCompare(b.surname));

    // Filter fields in users
    const filteredUsers = users.map((user) => {
      // Make a temporary partial structure so we can use setServiceCategories for translations
      const tempObj: Partial<Structure> = { categoriesDisplay: {} };

      // Translate offers
      const translatedOffers =
        user.personalOffersData?.map((offer) =>
          this.utils.setServiceCategories(this.categories, { ...offer, categoriesDisplay: {} }),
        ) || [];
      translatedOffers.forEach((offer) => {
        Object.entries(offer.categoriesDisplay).forEach(([categoryId, modules]) => {
          const category = this.categories.find((cat) => cat.id === categoryId);
          const categoryName = category?.name || categoryId;
          if (!tempObj.categoriesDisplay[categoryName]) {
            tempObj.categoriesDisplay[categoryName] = [];
          }
          if (Array.isArray(modules)) {
            tempObj.categoriesDisplay[categoryName] = [
              ...new Set([...tempObj.categoriesDisplay[categoryName], ...modules]),
            ];
          }
        });
      });

      // Construct the offer string
      const offre = Object.entries(tempObj.categoriesDisplay)
        .filter(([, modules]) => modules.length > 0)
        .map(([categoryName, modules]) => `${categoryName}: ${modules.map((m) => m.name).join(', ')}`)
        .join(' / ');

      // Construct the territories string
      const territoires = [
        ...new Set(
          user.structuresData?.flatMap((s) => {
            const foundCategory = this.categoriesCTMs[0].modules.find((c) => c.id === s.categories.ctm[0]);
            return foundCategory ? foundCategory.name : 'Nouveau Rhône';
          }),
        ),
      ].join(', ');

      return {
        Nom: `${user.name} ${user.surname.toUpperCase()}`,
        Fonction: user.job?.name,
        Téléphone: user.phone,
        Email: user.email,
        'Structure employeuse': user.employer?.name,
        'Commune(s)': [...new Set(user.structuresData?.map((s) => s.address.commune) || [])].join(', '),
        'Territoire(s)': territoires,
        "Structures d'intervention": user.structuresData?.map((s) => s.structureName).join(', '),
        Offre: offre,
      };
    });

    // Convert and download to CSV
    this.utils.convertAndDownloadCsv(filteredUsers, this.getFileBasename(dataType) + '.csv');

    this.isDownloading = false;
  }

  private getCTMs(): string[] {
    // If "Toute la métropole" choice, filter with all the CTMs
    if (this.selectedOption === 'choice2') {
      return this.categoriesCTMs[0].modules.map((ctm) => ctm.id);
    } else {
      // Filter with checked CTMs
      return this.checkedModulesCTMs.map((m) => m.id);
    }
  }

  private getFileBasename(dataType: DataType): string {
    const formattedDate = DateTime.now().toFormat('yyyy-MM-dd');
    return `resin-${dataType.slug}-${formattedDate}`;
  }

  private formatOpeningHoursForCSV(schedule: Week): string {
    const formattedSchedule = [];

    for (const { key, value } of schedule) {
      const translatedDay = WeekDayEnum[key];
      if (value.open && value.time.length > 0) {
        const times = value.time.map((t) => `${t.formatOpeningDate()} - ${t.formatClosingDate()}`).join(' / ');
        formattedSchedule.push(`${translatedDay}: ${times}`);
      } else {
        formattedSchedule.push(`${translatedDay}: fermé`);
      }
    }

    return formattedSchedule.join(', ');
  }

  public setFilters(checkedModules: Module[]): void {
    this.resetState();
    // Early return if no modules are checked
    if (checkedModules.length === 0) {
      this.resetFilters();
      return;
    }
    // Filter and set checked modules
    this.checkedModulesFilter = checkedModules.filter((module: Module) => !['choice3', 'choice4'].includes(module.id));
    // Set label for checked modules
    this.getResultTitle();
    // Determine module type and set corresponding property
    const firstModule = checkedModules[0];
    switch (firstModule.name) {
      case 'ctm':
        this.checkedModulesCTMs = checkedModules;
        this.selectedOption = 'choice3';
        break;
      case 'inseeCode':
        this.checkedModulesInseeCodes = checkedModules;
        this.selectedOption = 'choice4';
        break;
      default:
        if (firstModule.id.startsWith('choice')) {
          this.selectedOption = firstModule.id;
          this.checkedModulesTerritory = checkedModules;
        }
    }
    // Set territory select label
    const labelMap = {
      choice3: 'Territoires métropolitains',
      choice4: 'Communes',
    };
    this.territorySelectLabel = labelMap[this.selectedOption] || this.territorySelectLabel;
    // Set results visibility
    this.hasResults = this.checkedModulesFilter.length > 0;
  }

  private getResultTitle(): string {
    this.checkedModulesFilterLabel = this.checkedModulesFilter.map((m) => m.displayText).join(', ');
    return this.checkedModulesFilterLabel;
  }

  private resetState(): void {
    this.hasResults = false;
    this.checkedModulesCTMs = [];
    this.checkedModulesInseeCodes = [];
    this.checkedModulesFilter = [];
    this.checkedModulesFilterLabel = '';
    this.selectedOption = '';
  }

  public removeFilter(module: Module): void {
    const index: number = this.checkedModulesFilter.findIndex((m: Module) => m.id === module.id);
    if (index !== -1) {
      // update global list of checked filters
      this.checkedModulesFilter = this.checkedModulesFilter.filter((m: Module) => m.id !== module.id);
      // update each select
      this.checkedModulesCTMs = this.checkedModulesFilter.filter((module) => module.name === 'ctm');
      this.checkedModulesInseeCodes = this.checkedModulesFilter.filter((module) => module.name === 'inseeCode');
      this.getResultTitle();
      this.checkedModulesFilterLabel = this.checkedModulesFilter.length > 0 ? this.checkedModulesFilterLabel : '';
      this.hasResults = this.checkedModulesFilter.length > 0;
      if (this.checkedModulesFilter.length === 0) this.resetFilters();
    }
  }

  public resetFilters(): void {
    this.checkedModulesTerritory = [];
    this.territorySelectLabel = 'Sélectionnez un territoire';
    this.resetState();
  }

  private setMetaTags(appUrl: string): void {
    // Set meta tags (for linkedin and social networks publication)
    this.meta.updateTag({ name: 'title', content: this.title.getTitle() });
    this.meta.updateTag({ property: 'og:title', content: this.title.getTitle() });
    this.meta.updateTag({
      name: 'description',
      content:
        'Retrouvez ici les données présentes sur Rés’in, aux formats liste imprimable (pdf) et base de données (csv)',
    });
    this.meta.updateTag({
      property: 'og:description',
      content:
        'Retrouvez ici les données présentes sur Rés’in, aux formats liste imprimable (pdf) et base de données (csv)',
    });
    this.meta.updateTag({ property: 'og:type', content: 'website' });
    this.meta.updateTag({ name: 'image', content: `${appUrl}/assets/logos/resin-logo-data.png` });
    this.meta.updateTag({
      property: 'og:image',
      content: `${appUrl}/assets/logos/resin-logo-data.png`,
    });
    this.meta.updateTag({ property: 'og:url', content: `${appUrl}${this.router.url}` });
  }
}
